@@ -1,13 +1,22 @@
-from typing import Callable, Optional, TypeVar, Any
+from typing import Callable, Dict, Generic, Optional, TypeVar, Any
 from dataclasses import dataclass
 from inspect import signature, isgeneratorfunction, Parameter
 
 from .exceptions import GeneratorClosureException, InvalidGeneratorFactoryExcpetion
 
 
+def Depends(factory: Callable) -> Any:
+    return _Depends(factory=factory, lazy=False)
+
+
+def Lazy(factory: Callable) -> Any:
+    return _Depends(factory=factory, lazy=True)
+
+
 @dataclass(frozen=True)
-class Depends:
+class _Depends:
     factory: Callable
+    lazy: bool
 
 
 @dataclass(frozen=True)
@@ -16,13 +25,27 @@ class _Context:
     open_generators: list
 
 
-_resolver_cache = {}
+TLazyValue = TypeVar("TLazyValue")
+
+
+@dataclass
+class _LazyResolver(Generic[TLazyValue]):
+
+    factory: Callable[..., TLazyValue]
+    context: _Context
+
+    @property
+    def lazy(self) -> TLazyValue:
+        return _resolve(self.factory, self.context)
+
+
+_resolver_cache: Dict[Callable, Callable[[_Context], Any]] = {}
 
 
 TReturn = TypeVar("TReturn")
 
 
-def resolve(factory: Callable[[Any], TReturn], overrides: dict = None) -> TReturn:
+def resolve(factory: Callable[..., TReturn], overrides: dict = None) -> TReturn:
 
     if overrides is not None:
         resolved_cache = overrides
@@ -63,7 +86,7 @@ def _close_open_generators(context: _Context, resolve_exception: Optional[Except
             if exceptions is None:
                 exceptions = [ex]
             else:
-                exceptions.append[ex]
+                exceptions.append(ex)
 
     if exceptions is not None:
         raise GeneratorClosureException(
@@ -72,11 +95,11 @@ def _close_open_generators(context: _Context, resolve_exception: Optional[Except
         )
 
 
-def _resolve(factory: Callable[[Any], TReturn], context: _Context) -> TReturn:
+def _resolve(factory: Callable[..., TReturn], context: _Context) -> TReturn:
 
-    _resolver = _resolver_cache.get(factory)
-    if _resolver is not None:
-        return _resolver(context)
+    _cached_resolver = _resolver_cache.get(factory)
+    if _cached_resolver is not None:
+        return _cached_resolver(context)
 
     _signature = signature(factory)
 
@@ -85,39 +108,46 @@ def _resolve(factory: Callable[[Any], TReturn], context: _Context) -> TReturn:
     for parameter in _signature.parameters.values():
 
         if parameter.default is Parameter.empty:
-            raise Exception("Factory method missing default definition")
+            raise ValueError(
+                "Factory method missing default definition. "
+                f"{parameter.name=} {factory=}"
+            )
 
-        if not isinstance(parameter.default, Depends):
+        if not isinstance(parameter.default, _Depends):
             continue
 
-        _depends: Depends = parameter.default
-
-        dependency_factories.append((parameter.name, _depends.factory))
+        dependency_factories.append((parameter.name, parameter.default))
 
     is_generator = isgeneratorfunction(factory)
 
-    def _resolver(_context: _Context):
+    def _resolver(_context: _Context) -> TReturn:
+
+        context_value = _context.resolved_cache.get(factory)
+        if context_value is not None:
+            return context_value
 
         dependencies = {}
 
-        for name, _factory in dependency_factories:
+        for name, depends in dependency_factories:
 
-            context_value = _context.resolved_cache.get(_factory)
-            if context_value is not None:
-                dependencies[name] = context_value
+            if depends.lazy:
+                dependencies[name] = _LazyResolver(
+                    factory=depends.factory,
+                    context=_context,
+                )
                 continue
 
-            dependency = _resolve(_factory, _context)
-
-            dependencies[name] = dependency
-            _context.resolved_cache[_factory] = dependency
+            dependencies[name] = _resolve(depends.factory, _context)
 
         value = factory(**dependencies)
 
         if is_generator:
+
             _context.open_generators.append(value)
             value = next(value)
 
+        _context.resolved_cache[factory] = value
+
         return value
 
     _resolver_cache[factory] = _resolver
